﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;

using Microsoft.VisualStudio.Progression;
using Microsoft.VisualStudio.Progression.GraphModel;

using Qetza.VisualStudio.Analyzers.Properties;


namespace Qetza.VisualStudio.Analyzers
{
    /// <summary>
    /// The Name analyzer searches the graph linearly for nodes with Labels that start with "Qetza".
    /// </summary>
    [Provider(Name = "NameAnalyzer")]
    public class NameAnalyzer : IProvider
    {
        #region Fields

        // action context.
        private ActionContext m_context;

        // current graph.
        private Graph m_graph;

        // graph property to store IsNamed data.
        static GraphProperty IsNamed = GraphProperty.Register(
            "IsNamed",
            typeof(bool),
            new GraphMetadata(
                Resources.IsNamed,
                Resources.IsNamedDescription,
                null,
                GraphMetadataFlags.Default
                ),
            typeof(NameAnalyzer)
            );

        #endregion

        #region Methods

        /// <summary>
        /// Handle the analyze names action.
        /// </summary>
        public void OnAnalyzeNamed(ActionContext context)
        {
            this.m_graph = context.Graph;
            this.m_context = context;

            var result = context.OutputObjects;

            using (GraphTransactionScope scope = new GraphTransactionScope("NamedAnalyzer"))
            {
                // reset any prior name information.
                this.ClearNamed();
                this.ThrowIfCancelled();

                // analyzse nodes
                foreach (Node node in this.m_graph.VisibleNodes)
                {
                    if (node.Label.StartsWith("Qetza."))
                    {
                        node[IsNamed] = true;
                        result.Add(node);
                    }

                    this.ThrowIfCancelled();
                }

                scope.Complete();
            }
        }

        /// <summary>
        /// Check to see if analyzer is cancelled.
        /// </summary>
        public void ThrowIfCancelled()
        {
            if (this.m_context.Cancel)
            {
                throw new OperationCanceledException();
            }
        }

        #endregion

        #region IProvider Members

        /// <summary>
        /// Initialize the named analyzer
        /// </summary>
        /// <param name="serviceProvider">Not Used</param>
        public void Initialize(IServiceProvider serviceProvider)
        {
            var action = Microsoft.VisualStudio.Progression.Action.Register(
                "AnalyzeNamed",
                Resources.FindNamed,
                ActionFlags.IsAnalyzer | ActionFlags.IsGlobal,
                null
                );
            action[DgmlProperties.AnalyzerAnnotation] = IsNamed;
            action[DgmlProperties.AnalyzerStyle] = string.Format(
                CultureInfo.CurrentCulture,
                @"<Style TargetType='Node' xmlns='http://schemas.microsoft.com/vs/2009/dgml' GroupLabel='{0}' ValueLabel='True'>
                        <Condition Expression='{1}' />
                        <Setter Property='Background' Value='#FF008080' />
                    </Style>",
                Resources.IsNamed.Replace("'", "&apos;"),
                IsNamed.Id
                );
            action.ActionHandlers.Add(this.OnAnalyzeNamed);
            action.EnabledChanged += this.OnEnabledChanged;
            action[DgmlProperties.BrowserGroup] = Microsoft.VisualStudio.Progression.Global.Resources.BrowserGroup_ActionAnalysis_Id_NOLOC_;
        }

        /// <summary>
        /// Describes the types of node and link categories produced by this provider,
        /// where each Node in the graph is the Id of a Node Category and each link between the
        /// nodes describes the type of navigations possible between nodes.
        /// </summary>
        public Graph Schema { get { return null; } }

        #endregion

        #region Private Helpers

        /// <summary>
        /// Handle event when analyzer is enabled/disabled.
        /// </summary>
        private void OnEnabledChanged(object sender, EventArgs e)
        {
            if (this.m_graph != null)
            {
                using (GraphTransactionScope scope = new GraphTransactionScope("DisableUndo"))
                {
                    // we always clear names whether we are enabled or disabled, since OnAnalyzeNames will always re-do the names anyway.
                    this.ClearNamed();

                    scope.Complete();
                }
            }
        }

        /// <summary>
        /// Clear the property
        /// </summary>
        private void ClearNamed()
        {
            foreach (Node node in this.m_graph.Nodes)
            {
                if (node.HasValue(IsNamed))
                {
                    node.ClearValue(IsNamed);
                }

            }
        }

        #endregion

    }
}
